import Cookies from 'js-cookie'
import {v4 as uuid} from 'uuid'
import {getAdobeMarketingCloudVisitorIdFromWindow} from './adobeRepository'

const LOCALHOST = 'localhost'
const IJ_VISITOR_COOKIE_NAME = 'IJ_MCMID'

function getDomain({host = window.location.host}) {
  if (host.includes(LOCALHOST)) return LOCALHOST
  return host
    .split('.')
    .slice(-2)
    .join('.')
}

export function getVisitorIdCallback({
  host,
  getCookie = Cookies.get,
  setCookie = Cookies.set,
  getMarketingCloudVisitorId = getAdobeMarketingCloudVisitorIdFromWindow
}) {
  return () => {
    if (window.__mpi.visitorId) return Promise.resolve(window.__mpi.visitorId)

    // useractioncomplete when the user accepts or declines the consents
    // tcloaded when the user did already accept or decline the consents
    return new Promise(resolve => {
      window.__tcfapi('addEventListener', 2, async ({eventStatus}) => {
        const isEvent = ['useractioncomplete', 'tcloaded'].includes(eventStatus)
        if (!isEvent) {
          return Promise.resolve()
        }

        let visitorId = getCookie(IJ_VISITOR_COOKIE_NAME)
        if (!visitorId) {
          try {
            visitorId = await getMarketingCloudVisitorId()
          } catch (error) {
            console.error(error)
          }
        }

        if (!visitorId) {
          visitorId = uuid()
        }

        // Domain must be saved without `www` in order to be saved as .infojobs.net
        const domain = getDomain({host})
        setCookie(IJ_VISITOR_COOKIE_NAME, visitorId, {
          expires: 365,
          domain
        })
        window.__mpi.visitorId = visitorId
        return resolve(visitorId)
      })
    })
  }
}

function getSite({host = window.location.host}) {
  return host.indexOf('infojobs.it') > -1 ? 'infojobs.it' : 'infojobs.net'
}

export default dependencies => {
  // will be accessed by the segment-wrapper
  window.__mpi = {
    segmentWrapper: {
      facebookPixelId: 283351338713870,
      googleAdsPixelId: 1024323628,
      defaultProperties: {
        site: getSite(dependencies),
        vertical: 'jobs'
      },
      defaultContext: {
        site: getSite(dependencies)
      },
      getCustomAdobeVisitorId: getVisitorIdCallback(dependencies),
      tcfTrackDefaultProperties: {
        vertical: 'jobs'
      }
    }
  }
}
